/*
 * freetrans - comm.c
 * 	version 0.1
 * 	
 * 	Common TCP/IP Communication
 * 
 * Copyright (C) 2004, 2005, 2006  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netdb.h>
#include <netinet/in.h>
#include <comm.h>

int pcomm_open(const char *hostname, int port, int udp)
{
	struct sockaddr_in rem_addr;
	struct hostent *host;
	int fd, ret;

	if (udp)
		fd = socket(PF_INET, SOCK_DGRAM,
				getprotobyname("udp")->p_proto);
	else 
		fd = socket(PF_INET, SOCK_STREAM,
				getprotobyname("tcp")->p_proto);
	if (fd < 0)
		return fd;
	memset(&rem_addr, 0, sizeof(rem_addr));
	host = gethostbyname(hostname);
	if (host == NULL)
		return -EINVAL;
	rem_addr.sin_family = host->h_addrtype;
	rem_addr.sin_port = htons(port);
	memcpy(&rem_addr.sin_addr, host->h_addr, host->h_length);
	
	ret = connect(fd, (struct sockaddr *) &rem_addr, sizeof(rem_addr));
	if (ret < 0)
		return ret;
	return fd;
}

int pcomm_close(int fd)
{
	return close(fd);
}

#ifndef HAVE_CFMAKERAW
static void cfmakeraw(struct termios *termios_p)
{
	termios_p->c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR
			| IGNCR | ICRNL | IXON);
	termios_p->c_oflag &= ~OPOST;
	termios_p->c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
	termios_p->c_cflag &= ~(CSIZE | PARENB);
	termios_p->c_cflag |= CS8;
}
#endif

void serial_set_speed(int fd, speed_t speed, int opts)
{
	struct termios mode;

	memset(&mode, 0, sizeof(struct termios));
	cfmakeraw(&mode);
	mode.c_cflag |= CLOCAL | CREAD | CS8;
	if (opts & COMM_PAR_PAR) {
		mode.c_cflag |= PARENB;
		if (COMM_PAR_EVEN)
			mode.c_cflag &= ~PARODD;
	}
	if (opts & COMM_PAR_CSTOPB)
		mode.c_cflag &= ~CSTOPB;
	mode.c_oflag = 0;
	cfsetispeed(&mode, speed);
	cfsetospeed(&mode, speed);
	tcflush(fd, TCIFLUSH);
	tcsetattr(fd, TCSANOW, &mode);
}

speed_t itospeed(int speed)
{
	switch (speed) {
		case 50: return B50;
		case 75: return B75;
		case 110: return B110;
		case 134: return B134;
		case 150: return B150;
		case 200: return B200;
		case 300: return B300;
		case 600: return B600;
		case 1200: return B1200;
		case 2400: return B2400;
		case 4800: return B4800;
		case 9600: return B9600;
		case 19200: return B19200;
		case 38400: return B38400;
		case 57600: return B57600;
		case 115200: return B115200;
		case 230400: return B230400;
		case 460800: return B460800;
	}
	return -EINVAL;
}
